package javax.swing.plaf.basic;

import java.awt.datatransfer.Transferable;
import javax.swing.KeyStroke;
import javax.swing.text.Position;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import sun.swing.SwingUtilities2;
import javax.swing.DefaultListSelectionModel;
import java.awt.event.ActionEvent;
import sun.swing.UIAction;
import java.beans.PropertyChangeEvent;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListSelectionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.MouseEvent;
import javax.swing.plaf.ComponentUI;
import java.awt.Font;
import javax.swing.LookAndFeel;
import javax.swing.plaf.UIResource;
import javax.swing.InputMap;
import javax.swing.SwingUtilities;
import java.awt.Dimension;
import javax.swing.UIDefaults;
import javax.swing.DefaultListCellRenderer;
import javax.swing.UIManager;
import java.awt.Insets;
import java.awt.Point;
import java.awt.geom.Point2D;
import java.awt.Color;
import sun.swing.DefaultLookup;
import java.awt.Shape;
import javax.swing.JComponent;
import java.awt.Component;
import javax.swing.ListSelectionModel;
import javax.swing.ListModel;
import javax.swing.ListCellRenderer;
import java.awt.Rectangle;
import java.awt.Graphics;
import javax.swing.TransferHandler;
import java.beans.PropertyChangeListener;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.MouseInputListener;
import java.awt.event.FocusListener;
import javax.swing.CellRendererPane;
import javax.swing.JList;
import javax.swing.plaf.ListUI;

public class BasicListUI extends ListUI
{
    private static final StringBuilder BASELINE_COMPONENT_KEY;
    protected JList list;
    protected CellRendererPane rendererPane;
    protected FocusListener focusListener;
    protected MouseInputListener mouseInputListener;
    protected ListSelectionListener listSelectionListener;
    protected ListDataListener listDataListener;
    protected PropertyChangeListener propertyChangeListener;
    private BasicListUI.Handler handler;
    protected int[] cellHeights;
    protected int cellHeight;
    protected int cellWidth;
    protected int updateLayoutStateNeeded;
    private int listHeight;
    private int listWidth;
    private int layoutOrientation;
    private int columnCount;
    private int preferredHeight;
    private int rowsPerColumn;
    private long timeFactor;
    private boolean isFileList;
    private boolean isLeftToRight;
    protected static final int modelChanged = 1;
    protected static final int selectionModelChanged = 2;
    protected static final int fontChanged = 4;
    protected static final int fixedCellWidthChanged = 8;
    protected static final int fixedCellHeightChanged = 16;
    protected static final int prototypeCellValueChanged = 32;
    protected static final int cellRendererChanged = 64;
    private static final int layoutOrientationChanged = 128;
    private static final int heightChanged = 256;
    private static final int widthChanged = 512;
    private static final int componentOrientationChanged = 1024;
    private static final int DROP_LINE_THICKNESS = 2;
    private static final int CHANGE_LEAD = 0;
    private static final int CHANGE_SELECTION = 1;
    private static final int EXTEND_SELECTION = 2;
    private static final TransferHandler defaultTransferHandler;
    
    public BasicListUI() {
        this.list = null;
        this.cellHeights = null;
        this.cellHeight = -1;
        this.cellWidth = -1;
        this.updateLayoutStateNeeded = 1;
        this.timeFactor = 1000L;
        this.isFileList = false;
        this.isLeftToRight = true;
    }
    
    static void loadActionMap(LazyActionMap lazyActionMap) {
        lazyActionMap.put(new BasicListUI.Actions("selectPreviousColumn"));
        lazyActionMap.put(new BasicListUI.Actions("selectPreviousColumnExtendSelection"));
        lazyActionMap.put(new BasicListUI.Actions("selectPreviousColumnChangeLead"));
        lazyActionMap.put(new BasicListUI.Actions("selectNextColumn"));
        lazyActionMap.put(new BasicListUI.Actions("selectNextColumnExtendSelection"));
        lazyActionMap.put(new BasicListUI.Actions("selectNextColumnChangeLead"));
        lazyActionMap.put(new BasicListUI.Actions("selectPreviousRow"));
        lazyActionMap.put(new BasicListUI.Actions("selectPreviousRowExtendSelection"));
        lazyActionMap.put(new BasicListUI.Actions("selectPreviousRowChangeLead"));
        lazyActionMap.put(new BasicListUI.Actions("selectNextRow"));
        lazyActionMap.put(new BasicListUI.Actions("selectNextRowExtendSelection"));
        lazyActionMap.put(new BasicListUI.Actions("selectNextRowChangeLead"));
        lazyActionMap.put(new BasicListUI.Actions("selectFirstRow"));
        lazyActionMap.put(new BasicListUI.Actions("selectFirstRowExtendSelection"));
        lazyActionMap.put(new BasicListUI.Actions("selectFirstRowChangeLead"));
        lazyActionMap.put(new BasicListUI.Actions("selectLastRow"));
        lazyActionMap.put(new BasicListUI.Actions("selectLastRowExtendSelection"));
        lazyActionMap.put(new BasicListUI.Actions("selectLastRowChangeLead"));
        lazyActionMap.put(new BasicListUI.Actions("scrollUp"));
        lazyActionMap.put(new BasicListUI.Actions("scrollUpExtendSelection"));
        lazyActionMap.put(new BasicListUI.Actions("scrollUpChangeLead"));
        lazyActionMap.put(new BasicListUI.Actions("scrollDown"));
        lazyActionMap.put(new BasicListUI.Actions("scrollDownExtendSelection"));
        lazyActionMap.put(new BasicListUI.Actions("scrollDownChangeLead"));
        lazyActionMap.put(new BasicListUI.Actions("selectAll"));
        lazyActionMap.put(new BasicListUI.Actions("clearSelection"));
        lazyActionMap.put(new BasicListUI.Actions("addToSelection"));
        lazyActionMap.put(new BasicListUI.Actions("toggleAndAnchor"));
        lazyActionMap.put(new BasicListUI.Actions("extendTo"));
        lazyActionMap.put(new BasicListUI.Actions("moveSelectionTo"));
        lazyActionMap.put(TransferHandler.getCutAction().getValue("Name"), TransferHandler.getCutAction());
        lazyActionMap.put(TransferHandler.getCopyAction().getValue("Name"), TransferHandler.getCopyAction());
        lazyActionMap.put(TransferHandler.getPasteAction().getValue("Name"), TransferHandler.getPasteAction());
    }
    
    protected void paintCell(Graphics g, int n, Rectangle rectangle, ListCellRenderer listCellRenderer, ListModel listModel, ListSelectionModel listSelectionModel, int n2) {
        Component listCellRendererComponent = listCellRenderer.getListCellRendererComponent(this.list, listModel.getElementAt(n), n, listSelectionModel.isSelectedIndex(n), this.list.hasFocus() && n == n2);
        int x = rectangle.x;
        int y = rectangle.y;
        int width = rectangle.width;
        int height = rectangle.height;
        if (this.isFileList) {
            int min = Math.min(width, listCellRendererComponent.getPreferredSize().width + 4);
            if (!this.isLeftToRight) {
                x += width - min;
            }
            width = min;
        }
        this.rendererPane.paintComponent(g, listCellRendererComponent, this.list, x, y, width, height, true);
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        Shape clip = graphics.getClip();
        this.paintImpl(graphics, component);
        graphics.setClip(clip);
        this.paintDropLine(graphics);
    }
    
    private void paintImpl(Graphics graphics, JComponent component) {
        switch (this.layoutOrientation) {
            case 1: {
                if (this.list.getHeight() != this.listHeight) {
                    this.updateLayoutStateNeeded |= 0x100;
                    this.redrawList();
                    break;
                }
                break;
            }
            case 2: {
                if (this.list.getWidth() != this.listWidth) {
                    this.updateLayoutStateNeeded |= 0x200;
                    this.redrawList();
                    break;
                }
                break;
            }
        }
        this.maybeUpdateLayoutState();
        ListCellRenderer<Object> cellRenderer = this.list.getCellRenderer();
        ListModel<Object> model = this.list.getModel();
        ListSelectionModel selectionModel = this.list.getSelectionModel();
        int size;
        if (cellRenderer == null || (size = model.getSize()) == 0) {
            return;
        }
        Rectangle clipBounds = graphics.getClipBounds();
        int i;
        int n;
        if (component.getComponentOrientation().isLeftToRight()) {
            i = this.convertLocationToColumn(clipBounds.x, clipBounds.y);
            n = this.convertLocationToColumn(clipBounds.x + clipBounds.width, clipBounds.y);
        }
        else {
            i = this.convertLocationToColumn(clipBounds.x + clipBounds.width, clipBounds.y);
            n = this.convertLocationToColumn(clipBounds.x, clipBounds.y);
        }
        int n2 = clipBounds.y + clipBounds.height;
        int adjustIndex = adjustIndex(this.list.getLeadSelectionIndex(), this.list);
        int n3 = (this.layoutOrientation == 2) ? this.columnCount : 1;
        while (i <= n) {
            int convertLocationToRowInColumn = this.convertLocationToRowInColumn(clipBounds.y, i);
            int rowCount = this.getRowCount(i);
            int modelIndex = this.getModelIndex(i, convertLocationToRowInColumn);
            Rectangle cellBounds = this.getCellBounds(this.list, modelIndex, modelIndex);
            if (cellBounds == null) {
                return;
            }
            while (convertLocationToRowInColumn < rowCount && cellBounds.y < n2 && modelIndex < size) {
                cellBounds.height = this.getHeight(i, convertLocationToRowInColumn);
                graphics.setClip(cellBounds.x, cellBounds.y, cellBounds.width, cellBounds.height);
                graphics.clipRect(clipBounds.x, clipBounds.y, clipBounds.width, clipBounds.height);
                this.paintCell(graphics, modelIndex, cellBounds, cellRenderer, model, selectionModel, adjustIndex);
                cellBounds.y += cellBounds.height;
                modelIndex += n3;
                ++convertLocationToRowInColumn;
            }
            ++i;
        }
        this.rendererPane.removeAll();
        return;
    }
    
    private void paintDropLine(Graphics graphics) {
        JList.DropLocation dropLocation = this.list.getDropLocation();
        if (dropLocation == null || !dropLocation.isInsert()) {
            return;
        }
        Color color = DefaultLookup.getColor(this.list, this, "List.dropLineColor", null);
        if (color != null) {
            graphics.setColor(color);
            Rectangle dropLineRect = this.getDropLineRect(dropLocation);
            graphics.fillRect(dropLineRect.x, dropLineRect.y, dropLineRect.width, dropLineRect.height);
        }
        return;
    }
    
    private Rectangle getDropLineRect(JList.DropLocation dropLocation) {
        int size = this.list.getModel().getSize();
        if (size == 0) {
            Insets insets = this.list.getInsets();
            if (this.layoutOrientation == 2) {
                if (this.isLeftToRight) {
                    return new Rectangle(insets.left, insets.top, 2, 20);
                }
                return new Rectangle(this.list.getWidth() - 2 - insets.right, insets.top, 2, 20);
            }
            else {
                return new Rectangle(insets.left, insets.top, this.list.getWidth() - insets.left - insets.right, 2);
            }
        }
        else {
            int index = dropLocation.getIndex();
            int n = 0;
            Rectangle rectangle;
            if (this.layoutOrientation == 2) {
                if (index == size) {
                    n = 1;
                }
                else if (index != 0 && this.convertModelToRow(index) != this.convertModelToRow(index - 1)) {
                    Rectangle cellBounds3 = this.getCellBounds(this.list, index - 1);
                    Rectangle cellBounds4 = this.getCellBounds(this.list, index);
                    Point dropPoint2 = dropLocation.getDropPoint();
                    if (this.isLeftToRight) {
                        n = (Point2D.distance((double)(cellBounds3.x + cellBounds3.width), (double)(cellBounds3.y + (int)((double)cellBounds3.height / 2.0)), (double)dropPoint2.x, (double)dropPoint2.y) < Point2D.distance((double)cellBounds4.x, (double)(cellBounds4.y + (int)((double)cellBounds4.height / 2.0)), (double)dropPoint2.x, (double)dropPoint2.y));
                    }
                    else {
                        n = (Point2D.distance((double)cellBounds3.x, (double)(cellBounds3.y + (int)((double)cellBounds3.height / 2.0)), (double)dropPoint2.x, (double)dropPoint2.y) < Point2D.distance((double)(cellBounds4.x + cellBounds4.width), (double)(cellBounds4.y + (int)((double)cellBounds3.height / 2.0)), (double)dropPoint2.x, (double)dropPoint2.y));
                    }
                }
                if (n != 0) {
                    --index;
                    rectangle = this.getCellBounds(this.list, index);
                    if (this.isLeftToRight) {
                        rectangle.x += rectangle.width;
                    }
                    else {
                        rectangle.x -= 2;
                    }
                }
                else {
                    rectangle = this.getCellBounds(this.list, index);
                    if (!this.isLeftToRight) {
                        rectangle.x += rectangle.width - 2;
                    }
                }
                if (rectangle.x >= this.list.getWidth()) {
                    rectangle.x = this.list.getWidth() - 2;
                }
                else if (rectangle.x < 0) {
                    rectangle.x = 0;
                }
                rectangle.width = 2;
            }
            else if (this.layoutOrientation == 1) {
                if (index == size) {
                    --index;
                    rectangle = this.getCellBounds(this.list, index);
                    rectangle.y += rectangle.height;
                }
                else if (index != 0 && this.convertModelToColumn(index) != this.convertModelToColumn(index - 1)) {
                    Rectangle cellBounds = this.getCellBounds(this.list, index - 1);
                    Rectangle cellBounds2 = this.getCellBounds(this.list, index);
                    Point dropPoint = dropLocation.getDropPoint();
                    if (Point2D.distance((double)(cellBounds.x + (int)((double)cellBounds.width / 2.0)), (double)(cellBounds.y + cellBounds.height), (double)dropPoint.x, (double)dropPoint.y) < Point2D.distance((double)(cellBounds2.x + (int)((double)cellBounds2.width / 2.0)), (double)cellBounds2.y, (double)dropPoint.x, (double)dropPoint.y)) {
                        --index;
                        rectangle = this.getCellBounds(this.list, index);
                        rectangle.y += rectangle.height;
                    }
                    else {
                        rectangle = this.getCellBounds(this.list, index);
                    }
                }
                else {
                    rectangle = this.getCellBounds(this.list, index);
                }
                if (rectangle.y >= this.list.getHeight()) {
                    rectangle.y = this.list.getHeight() - 2;
                }
                rectangle.height = 2;
            }
            else {
                if (index == size) {
                    --index;
                    rectangle = this.getCellBounds(this.list, index);
                    rectangle.y += rectangle.height;
                }
                else {
                    rectangle = this.getCellBounds(this.list, index);
                }
                if (rectangle.y >= this.list.getHeight()) {
                    rectangle.y = this.list.getHeight() - 2;
                }
                rectangle.height = 2;
            }
            return rectangle;
        }
    }
    
    @Override
    public int getBaseline(JComponent c, int width, int height) {
        super.getBaseline(c, width, height);
        int height2 = this.list.getFixedCellHeight();
        UIDefaults lookAndFeelDefaults = UIManager.getLookAndFeelDefaults();
        Component listCellRendererComponent = (Component)lookAndFeelDefaults.get(BasicListUI.BASELINE_COMPONENT_KEY);
        if (listCellRendererComponent == null) {
            ListCellRenderer listCellRenderer = (ListCellRenderer)UIManager.get("List.cellRenderer");
            if (listCellRenderer == null) {
                listCellRenderer = new DefaultListCellRenderer();
            }
            listCellRendererComponent = listCellRenderer.getListCellRendererComponent(this.list, "a", -1, false, false);
            lookAndFeelDefaults.put(BasicListUI.BASELINE_COMPONENT_KEY, listCellRendererComponent);
        }
        listCellRendererComponent.setFont(this.list.getFont());
        if (height2 == -1) {
            height2 = listCellRendererComponent.getPreferredSize().height;
        }
        return listCellRendererComponent.getBaseline(Integer.MAX_VALUE, height2) + this.list.getInsets().top;
    }
    
    @Override
    public Component.BaselineResizeBehavior getBaselineResizeBehavior(JComponent c) {
        super.getBaselineResizeBehavior(c);
        return Component.BaselineResizeBehavior.CONSTANT_ASCENT;
    }
    
    @Override
    public Dimension getPreferredSize(JComponent component) {
        this.maybeUpdateLayoutState();
        int n = this.list.getModel().getSize() - 1;
        if (n < 0) {
            return new Dimension(0, 0);
        }
        Insets insets = this.list.getInsets();
        int width = this.cellWidth * this.columnCount + insets.left + insets.right;
        int preferredHeight;
        if (this.layoutOrientation != 0) {
            preferredHeight = this.preferredHeight;
        }
        else {
            Rectangle cellBounds = this.getCellBounds(this.list, n);
            if (cellBounds != null) {
                preferredHeight = cellBounds.y + cellBounds.height + insets.bottom;
            }
            else {
                preferredHeight = 0;
            }
        }
        return new Dimension(width, preferredHeight);
    }
    
    protected void selectPreviousIndex() {
        int selectedIndex = this.list.getSelectedIndex();
        if (selectedIndex > 0) {
            --selectedIndex;
            this.list.setSelectedIndex(selectedIndex);
            this.list.ensureIndexIsVisible(selectedIndex);
        }
    }
    
    protected void selectNextIndex() {
        int selectedIndex = this.list.getSelectedIndex();
        if (selectedIndex + 1 < this.list.getModel().getSize()) {
            ++selectedIndex;
            this.list.setSelectedIndex(selectedIndex);
            this.list.ensureIndexIsVisible(selectedIndex);
        }
    }
    
    protected void installKeyboardActions() {
        SwingUtilities.replaceUIInputMap(this.list, 0, this.getInputMap(0));
        LazyActionMap.installLazyActionMap(this.list, BasicListUI.class, "List.actionMap");
    }
    
    InputMap getInputMap(int n) {
        if (n == 0) {
            InputMap parent = (InputMap)DefaultLookup.get(this.list, this, "List.focusInputMap");
            InputMap inputMap;
            if (this.isLeftToRight || (inputMap = (InputMap)DefaultLookup.get(this.list, this, "List.focusInputMap.RightToLeft")) == null) {
                return parent;
            }
            inputMap.setParent(parent);
            return inputMap;
        }
        else {
            return null;
        }
    }
    
    protected void uninstallKeyboardActions() {
        SwingUtilities.replaceUIActionMap(this.list, null);
        SwingUtilities.replaceUIInputMap(this.list, 0, null);
    }
    
    protected void installListeners() {
        TransferHandler transferHandler = this.list.getTransferHandler();
        if (transferHandler == null || transferHandler instanceof UIResource) {
            this.list.setTransferHandler(BasicListUI.defaultTransferHandler);
            if ((this.list.getDropTarget() instanceof UIResource)) {
                this.list.setDropTarget(null);
            }
        }
        this.focusListener = this.createFocusListener();
        this.mouseInputListener = this.createMouseInputListener();
        this.propertyChangeListener = this.createPropertyChangeListener();
        this.listSelectionListener = this.createListSelectionListener();
        this.listDataListener = this.createListDataListener();
        this.list.addFocusListener(this.focusListener);
        this.list.addMouseListener(this.mouseInputListener);
        this.list.addMouseMotionListener(this.mouseInputListener);
        this.list.addPropertyChangeListener(this.propertyChangeListener);
        this.list.addKeyListener(this.getHandler());
        ListModel<Object> model = this.list.getModel();
        if (model != null) {
            model.addListDataListener(this.listDataListener);
        }
        ListSelectionModel selectionModel = this.list.getSelectionModel();
        if (selectionModel != null) {
            selectionModel.addListSelectionListener(this.listSelectionListener);
        }
    }
    
    protected void uninstallListeners() {
        this.list.removeFocusListener(this.focusListener);
        this.list.removeMouseListener(this.mouseInputListener);
        this.list.removeMouseMotionListener(this.mouseInputListener);
        this.list.removePropertyChangeListener(this.propertyChangeListener);
        this.list.removeKeyListener(this.getHandler());
        ListModel<Object> model = this.list.getModel();
        if (model != null) {
            model.removeListDataListener(this.listDataListener);
        }
        ListSelectionModel selectionModel = this.list.getSelectionModel();
        if (selectionModel != null) {
            selectionModel.removeListSelectionListener(this.listSelectionListener);
        }
        this.focusListener = null;
        this.mouseInputListener = null;
        this.listSelectionListener = null;
        this.listDataListener = null;
        this.propertyChangeListener = null;
        this.handler = null;
    }
    
    protected void installDefaults() {
        this.list.setLayout(null);
        LookAndFeel.installBorder(this.list, "List.border");
        LookAndFeel.installColorsAndFont(this.list, "List.background", "List.foreground", "List.font");
        LookAndFeel.installProperty(this.list, "opaque", Boolean.TRUE);
        if (this.list.getCellRenderer() == null) {
            this.list.setCellRenderer((ListCellRenderer)(ListCellRenderer)UIManager.get("List.cellRenderer"));
        }
        Color selectionBackground = this.list.getSelectionBackground();
        if (selectionBackground == null || selectionBackground instanceof UIResource) {
            this.list.setSelectionBackground(UIManager.getColor("List.selectionBackground"));
        }
        Color selectionForeground = this.list.getSelectionForeground();
        if (selectionForeground == null || selectionForeground instanceof UIResource) {
            this.list.setSelectionForeground(UIManager.getColor("List.selectionForeground"));
        }
        Long n = (Long)UIManager.get("List.timeFactor");
        this.timeFactor = ((n != null) ? ((long)n) : 1000L);
        this.updateIsFileList();
    }
    
    private void updateIsFileList() {
        int equals = Boolean.TRUE.equals(this.list.getClientProperty("List.isFileList"));
        if (equals != (this.isFileList ? 1 : 0)) {
            this.isFileList = equals;
            Font font = this.list.getFont();
            if (font == null || font instanceof UIResource) {
                Font font2 = UIManager.getFont(equals ? "FileChooser.listFont" : "List.font");
                if (font2 != null && font2 != font) {
                    this.list.setFont(font2);
                }
            }
        }
    }
    
    protected void uninstallDefaults() {
        LookAndFeel.uninstallBorder(this.list);
        if ((this.list.getFont() instanceof UIResource)) {
            this.list.setFont(null);
        }
        if ((this.list.getForeground() instanceof UIResource)) {
            this.list.setForeground(null);
        }
        if ((this.list.getBackground() instanceof UIResource)) {
            this.list.setBackground(null);
        }
        if ((this.list.getSelectionBackground() instanceof UIResource)) {
            this.list.setSelectionBackground(null);
        }
        if ((this.list.getSelectionForeground() instanceof UIResource)) {
            this.list.setSelectionForeground(null);
        }
        if ((this.list.getCellRenderer() instanceof UIResource)) {
            this.list.setCellRenderer(null);
        }
        if ((this.list.getTransferHandler() instanceof UIResource)) {
            this.list.setTransferHandler(null);
        }
    }
    
    @Override
    public void installUI(JComponent component) {
        this.list = (JList)component;
        this.layoutOrientation = this.list.getLayoutOrientation();
        this.rendererPane = new CellRendererPane();
        this.list.add(this.rendererPane);
        this.columnCount = 1;
        this.updateLayoutStateNeeded = 1;
        this.isLeftToRight = this.list.getComponentOrientation().isLeftToRight();
        this.installDefaults();
        this.installListeners();
        this.installKeyboardActions();
    }
    
    @Override
    public void uninstallUI(JComponent component) {
        this.uninstallListeners();
        this.uninstallDefaults();
        this.uninstallKeyboardActions();
        int n = -1;
        this.cellHeight = n;
        this.cellWidth = n;
        this.cellHeights = null;
        int n2 = -1;
        this.listHeight = n2;
        this.listWidth = n2;
        this.list.remove(this.rendererPane);
        this.rendererPane = null;
        this.list = null;
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new BasicListUI();
    }
    
    @Override
    public int locationToIndex(JList list, Point point) {
        this.maybeUpdateLayoutState();
        return this.convertLocationToModel(point.x, point.y);
    }
    
    @Override
    public Point indexToLocation(JList list, int n) {
        this.maybeUpdateLayoutState();
        Rectangle cellBounds = this.getCellBounds(list, n, n);
        if (cellBounds != null) {
            return new Point(cellBounds.x, cellBounds.y);
        }
        return null;
    }
    
    @Override
    public Rectangle getCellBounds(JList list, int n, int n2) {
        this.maybeUpdateLayoutState();
        int min = Math.min(n, n2);
        int max = Math.max(n, n2);
        if (min >= list.getModel().getSize()) {
            return null;
        }
        Rectangle cellBounds = this.getCellBounds(list, min);
        if (cellBounds == null) {
            return null;
        }
        if (min == max) {
            return cellBounds;
        }
        Rectangle cellBounds2 = this.getCellBounds(list, max);
        if (cellBounds2 != null) {
            if (this.layoutOrientation == 2) {
                if (this.convertModelToRow(min) != this.convertModelToRow(max)) {
                    cellBounds.x = 0;
                    cellBounds.width = list.getWidth();
                }
            }
            else if (cellBounds.x != cellBounds2.x) {
                cellBounds.y = 0;
                cellBounds.height = list.getHeight();
            }
            cellBounds.add(cellBounds2);
            return cellBounds;
        }
        else {
            return cellBounds;
        }
    }
    
    private Rectangle getCellBounds(JList list, int n) {
        this.maybeUpdateLayoutState();
        int convertModelToRow = this.convertModelToRow(n);
        int convertModelToColumn = this.convertModelToColumn(n);
        if (convertModelToRow == -1 || convertModelToColumn == -1) {
            return null;
        }
        Insets insets = list.getInsets();
        int cellWidth = this.cellWidth;
        int top = insets.top;
        int left;
        int height;
        switch (this.layoutOrientation) {
            case 1:
            case 2: {
                if (this.isLeftToRight) {
                    left = insets.left + convertModelToColumn * this.cellWidth;
                }
                else {
                    left = list.getWidth() - insets.right - (convertModelToColumn + 1) * this.cellWidth;
                }
                top += this.cellHeight * convertModelToRow;
                height = this.cellHeight;
                break;
            }
            default: {
                left = insets.left;
                if (this.cellHeights == null) {
                    top += this.cellHeight * convertModelToRow;
                }
                else if (convertModelToRow >= this.cellHeights.length) {
                    top = 0;
                }
                else {
                    for (int i = 0; i < convertModelToRow; ++i) {
                        top += this.cellHeights[i];
                    }
                }
                cellWidth = list.getWidth() - (insets.left + insets.right);
                height = this.getRowHeight(n);
                break;
            }
        }
        return new Rectangle(left, top, cellWidth, height);
    }
    
    protected int getRowHeight(int n) {
        return this.getHeight(0, n);
    }
    
    protected int convertYToRow(int n) {
        return this.convertLocationToRow(0, n, false);
    }
    
    protected int convertRowToY(int n) {
        if (n >= this.getRowCount(0) || n < 0) {
            return -1;
        }
        return this.getCellBounds(this.list, n, n).y;
    }
    
    private int getHeight(int n, int n2) {
        if (n < 0 || n > this.columnCount || n2 < 0) {
            return -1;
        }
        if (this.layoutOrientation != 0) {
            return this.cellHeight;
        }
        if (n2 >= this.list.getModel().getSize()) {
            return -1;
        }
        return (this.cellHeights == null) ? this.cellHeight : ((n2 < this.cellHeights.length) ? this.cellHeights[n2] : -1);
    }
    
    private int convertLocationToRow(int n, int n2, boolean b) {
        int size = this.list.getModel().getSize();
        if (size <= 0) {
            return -1;
        }
        Insets insets = this.list.getInsets();
        if (this.cellHeights == null) {
            int n4 = (this.cellHeight == 0) ? 0 : ((n2 - insets.top) / this.cellHeight);
            if (b) {
                if (n4 < 0) {
                    n4 = 0;
                }
                else if (n4 >= size) {
                    n4 = size - 1;
                }
            }
            return n4;
        }
        else {
            if (size > this.cellHeights.length) {
                return -1;
            }
            int top = insets.top;
            int n3 = 0;
            if (b && n2 < top) {
                return 0;
            }
            int i = 0;
            while (i < size) {
                if (n2 >= top && n2 < top + this.cellHeights[i]) {
                    return n3;
                }
                top += this.cellHeights[i];
                ++n3;
                ++i;
            }
            return i - 1;
        }
    }
    
    private int convertLocationToRowInColumn(int n, int n2) {
        int n3 = 0;
        if (this.layoutOrientation != 0) {
            if (this.isLeftToRight) {
                n3 = n2 * this.cellWidth;
            }
            else {
                n3 = this.list.getWidth() - (n2 + 1) * this.cellWidth - this.list.getInsets().right;
            }
        }
        return this.convertLocationToRow(n3, n, true);
    }
    
    private int convertLocationToModel(int n, int n2) {
        int convertLocationToRow = this.convertLocationToRow(n, n2, true);
        int convertLocationToColumn = this.convertLocationToColumn(n, n2);
        if (convertLocationToRow >= 0 && convertLocationToColumn >= 0) {
            return this.getModelIndex(convertLocationToColumn, convertLocationToRow);
        }
        return -1;
    }
    
    private int getRowCount(int n) {
        if (n < 0 || n >= this.columnCount) {
            return -1;
        }
        if (this.layoutOrientation == 0 || (n == 0 && this.columnCount == 1)) {
            return this.list.getModel().getSize();
        }
        if (n >= this.columnCount) {
            return -1;
        }
        if (this.layoutOrientation == 1) {
            if (n < this.columnCount - 1) {
                return this.rowsPerColumn;
            }
            return this.list.getModel().getSize() - (this.columnCount - 1) * this.rowsPerColumn;
        }
        else {
            if (n >= this.columnCount - (this.columnCount * this.rowsPerColumn - this.list.getModel().getSize())) {
                return Math.max(0, this.rowsPerColumn - 1);
            }
            return this.rowsPerColumn;
        }
    }
    
    private int getModelIndex(int n, int a) {
        switch (this.layoutOrientation) {
            case 1: {
                return Math.min(this.list.getModel().getSize() - 1, this.rowsPerColumn * n + Math.min(a, this.rowsPerColumn - 1));
            }
            case 2: {
                return Math.min(this.list.getModel().getSize() - 1, a * this.columnCount + n);
            }
            default: {
                return a;
            }
        }
    }
    
    private int convertLocationToColumn(int n, int n2) {
        if (this.cellWidth > 0) {
            if (this.layoutOrientation == 0) {
                return 0;
            }
            Insets insets = this.list.getInsets();
            int n3;
            if (this.isLeftToRight) {
                n3 = (n - insets.left) / this.cellWidth;
            }
            else {
                n3 = (this.list.getWidth() - n - insets.right - 1) / this.cellWidth;
            }
            if (n3 < 0) {
                return 0;
            }
            if (n3 >= this.columnCount) {
                return this.columnCount - 1;
            }
            return n3;
        }
        else {
            return 0;
        }
    }
    
    private int convertModelToRow(int n) {
        int size = this.list.getModel().getSize();
        if (n < 0 || n >= size) {
            return -1;
        }
        if (this.layoutOrientation != 0 && this.columnCount > 1 && this.rowsPerColumn > 0) {
            if (this.layoutOrientation == 1) {
                return n % this.rowsPerColumn;
            }
            return n / this.columnCount;
        }
        else {
            return n;
        }
    }
    
    private int convertModelToColumn(int n) {
        int size = this.list.getModel().getSize();
        if (n < 0 || n >= size) {
            return -1;
        }
        if (this.layoutOrientation != 0 && this.rowsPerColumn > 0 && this.columnCount > 1) {
            if (this.layoutOrientation == 1) {
                return n / this.rowsPerColumn;
            }
            return n % this.columnCount;
        }
        else {
            return 0;
        }
    }
    
    protected void maybeUpdateLayoutState() {
        if (this.updateLayoutStateNeeded != 0) {
            this.updateLayoutState();
            this.updateLayoutStateNeeded = 0;
        }
    }
    
    protected void updateLayoutState() {
        int fixedCellHeight = this.list.getFixedCellHeight();
        int fixedCellWidth = this.list.getFixedCellWidth();
        this.cellWidth = ((fixedCellWidth != -1) ? fixedCellWidth : -1);
        if (fixedCellHeight != -1) {
            this.cellHeight = fixedCellHeight;
            this.cellHeights = null;
        }
        else {
            this.cellHeight = -1;
            this.cellHeights = new int[this.list.getModel().getSize()];
        }
        if (fixedCellWidth == -1 || fixedCellHeight == -1) {
            ListModel<Object> model = this.list.getModel();
            int size = model.getSize();
            ListCellRenderer cellRenderer = this.list.getCellRenderer();
            if (cellRenderer != null) {
                for (int j = 0; j < size; ++j) {
                    Component listCellRendererComponent = cellRenderer.getListCellRendererComponent(this.list, model.getElementAt(j), j, false, false);
                    this.rendererPane.add(listCellRendererComponent);
                    Dimension preferredSize = listCellRendererComponent.getPreferredSize();
                    if (fixedCellWidth == -1) {
                        this.cellWidth = Math.max(preferredSize.width, this.cellWidth);
                    }
                    if (fixedCellHeight == -1) {
                        this.cellHeights[j] = preferredSize.height;
                    }
                }
            }
            else {
                if (this.cellWidth == -1) {
                    this.cellWidth = 0;
                }
                if (this.cellHeights == null) {
                    this.cellHeights = new int[size];
                }
                for (int i = 0; i < size; ++i) {
                    this.cellHeights[i] = 0;
                }
            }
        }
        this.columnCount = 1;
        if (this.layoutOrientation != 0) {
            this.updateHorizontalLayoutState(fixedCellWidth, fixedCellHeight);
        }
    }
    
    private void updateHorizontalLayoutState(int n, int n2) {
        int visibleRowCount = this.list.getVisibleRowCount();
        int size = this.list.getModel().getSize();
        Insets insets = this.list.getInsets();
        this.listHeight = this.list.getHeight();
        this.listWidth = this.list.getWidth();
        if (size == 0) {
            int n3 = 0;
            this.columnCount = n3;
            this.rowsPerColumn = n3;
            this.preferredHeight = insets.top + insets.bottom;
            return;
        }
        if (n2 != -1) {
        }
        else {
            int max = 0;
            if (this.cellHeights.length > 0) {
                max = this.cellHeights[this.cellHeights.length - 1];
                for (int i = this.cellHeights.length - 2; i >= 0; --i) {
                    max = Math.max(max, this.cellHeights[i]);
                }
            }
            int cellHeight = max;
            this.cellHeight = cellHeight;
            n2 = cellHeight;
            this.cellHeights = null;
        }
        this.rowsPerColumn = size;
        if (visibleRowCount > 0) {
            this.rowsPerColumn = visibleRowCount;
            this.columnCount = Math.max(1, size / this.rowsPerColumn);
            if (size > 0 && size > this.rowsPerColumn && size % this.rowsPerColumn != 0) {
                ++this.columnCount;
            }
            if (this.layoutOrientation == 2) {
                this.rowsPerColumn = size / this.columnCount;
                if (size % this.columnCount > 0) {
                    ++this.rowsPerColumn;
                }
            }
        }
        else if (this.layoutOrientation == 1 && n2 != 0) {
            this.rowsPerColumn = Math.max(1, (this.listHeight - insets.top - insets.bottom) / n2);
            this.columnCount = Math.max(1, size / this.rowsPerColumn);
            if (size > 0 && size > this.rowsPerColumn && size % this.rowsPerColumn != 0) {
                ++this.columnCount;
            }
        }
        else if (this.layoutOrientation == 2 && this.cellWidth > 0 && this.listWidth > 0) {
            this.columnCount = Math.max(1, (this.listWidth - insets.left - insets.right) / this.cellWidth);
            this.rowsPerColumn = size / this.columnCount;
            if (size % this.columnCount > 0) {
                ++this.rowsPerColumn;
            }
        }
        this.preferredHeight = this.rowsPerColumn * this.cellHeight + insets.top + insets.bottom;
        return;
    }
    
    private BasicListUI.Handler getHandler() {
        if (this.handler == null) {
            this.handler = new BasicListUI.Handler();
        }
        return this.handler;
    }
    
    protected MouseInputListener createMouseInputListener() {
        return this.getHandler();
    }
    
    protected FocusListener createFocusListener() {
        return this.getHandler();
    }
    
    protected ListSelectionListener createListSelectionListener() {
        return this.getHandler();
    }
    
    private void redrawList() {
        this.list.revalidate();
        this.list.repaint();
    }
    
    protected ListDataListener createListDataListener() {
        return this.getHandler();
    }
    
    protected PropertyChangeListener createPropertyChangeListener() {
        return this.getHandler();
    }
    
    private static int adjustIndex(int n, JList list) {
        return (n < list.getModel().getSize()) ? n : -1;
    }
    
    static BasicListUI.Handler access$100(BasicListUI basicListUI) {
        return basicListUI.getHandler();
    }
    
    static int access$200(int n, JList list) {
        return adjustIndex(n, list);
    }
    
    static int access$400(BasicListUI basicListUI, int n) {
        return basicListUI.convertModelToColumn(n);
    }
    
    static int access$500(BasicListUI basicListUI, int n) {
        return basicListUI.convertModelToRow(n);
    }
    
    static int access$600(BasicListUI basicListUI, int n) {
        return basicListUI.getRowCount(n);
    }
    
    static int access$700(BasicListUI basicListUI, int n, int n2) {
        return basicListUI.getModelIndex(n, n2);
    }
    
    static void access$900(BasicListUI basicListUI) {
        basicListUI.redrawList();
    }
    
    static void access$1200(BasicListUI basicListUI) {
        basicListUI.updateIsFileList();
    }
    
    static Rectangle access$1300(BasicListUI basicListUI, JList.DropLocation dropLocation) {
        return basicListUI.getDropLineRect(dropLocation);
    }
    
    static Rectangle access$1400(BasicListUI basicListUI, JList list, int n) {
        return basicListUI.getCellBounds(list, n);
    }
    
    static {
        BASELINE_COMPONENT_KEY = new StringBuilder("List.baselineComponent");
        defaultTransferHandler = new BasicListUI.ListTransferHandler();
    }
    
    public class MouseInputHandler implements MouseInputListener
    {
        @Override
        public void mouseClicked(MouseEvent mouseEvent) {
            BasicListUI.access$100(BasicListUI.this).mouseClicked(mouseEvent);
        }
        
        @Override
        public void mouseEntered(MouseEvent mouseEvent) {
            BasicListUI.access$100(BasicListUI.this).mouseEntered(mouseEvent);
        }
        
        @Override
        public void mouseExited(MouseEvent mouseEvent) {
            BasicListUI.access$100(BasicListUI.this).mouseExited(mouseEvent);
        }
        
        @Override
        public void mousePressed(MouseEvent mouseEvent) {
            BasicListUI.access$100(BasicListUI.this).mousePressed(mouseEvent);
        }
        
        @Override
        public void mouseDragged(MouseEvent mouseEvent) {
            BasicListUI.access$100(BasicListUI.this).mouseDragged(mouseEvent);
        }
        
        @Override
        public void mouseMoved(MouseEvent mouseEvent) {
            BasicListUI.access$100(BasicListUI.this).mouseMoved(mouseEvent);
        }
        
        @Override
        public void mouseReleased(MouseEvent mouseEvent) {
            BasicListUI.access$100(BasicListUI.this).mouseReleased(mouseEvent);
        }
    }
    
    public class FocusHandler implements FocusListener
    {
        protected void repaintCellFocus() {
            BasicListUI.access$100(BasicListUI.this).repaintCellFocus();
        }
        
        @Override
        public void focusGained(FocusEvent focusEvent) {
            BasicListUI.access$100(BasicListUI.this).focusGained(focusEvent);
        }
        
        @Override
        public void focusLost(FocusEvent focusEvent) {
            BasicListUI.access$100(BasicListUI.this).focusLost(focusEvent);
        }
    }
    
    public class ListSelectionHandler implements ListSelectionListener
    {
        @Override
        public void valueChanged(ListSelectionEvent listSelectionEvent) {
            BasicListUI.access$100(BasicListUI.this).valueChanged(listSelectionEvent);
        }
    }
    
    public class ListDataHandler implements ListDataListener
    {
        @Override
        public void intervalAdded(ListDataEvent listDataEvent) {
            BasicListUI.access$100(BasicListUI.this).intervalAdded(listDataEvent);
        }
        
        @Override
        public void intervalRemoved(ListDataEvent listDataEvent) {
            BasicListUI.access$100(BasicListUI.this).intervalRemoved(listDataEvent);
        }
        
        @Override
        public void contentsChanged(ListDataEvent listDataEvent) {
            BasicListUI.access$100(BasicListUI.this).contentsChanged(listDataEvent);
        }
    }
    
    public class PropertyChangeHandler implements PropertyChangeListener
    {
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            BasicListUI.access$100(BasicListUI.this).propertyChange(propertyChangeEvent);
        }
    }
    
    private static class Actions extends UIAction
    {
        private static final String SELECT_PREVIOUS_COLUMN = "selectPreviousColumn";
        private static final String SELECT_PREVIOUS_COLUMN_EXTEND = "selectPreviousColumnExtendSelection";
        private static final String SELECT_PREVIOUS_COLUMN_CHANGE_LEAD = "selectPreviousColumnChangeLead";
        private static final String SELECT_NEXT_COLUMN = "selectNextColumn";
        private static final String SELECT_NEXT_COLUMN_EXTEND = "selectNextColumnExtendSelection";
        private static final String SELECT_NEXT_COLUMN_CHANGE_LEAD = "selectNextColumnChangeLead";
        private static final String SELECT_PREVIOUS_ROW = "selectPreviousRow";
        private static final String SELECT_PREVIOUS_ROW_EXTEND = "selectPreviousRowExtendSelection";
        private static final String SELECT_PREVIOUS_ROW_CHANGE_LEAD = "selectPreviousRowChangeLead";
        private static final String SELECT_NEXT_ROW = "selectNextRow";
        private static final String SELECT_NEXT_ROW_EXTEND = "selectNextRowExtendSelection";
        private static final String SELECT_NEXT_ROW_CHANGE_LEAD = "selectNextRowChangeLead";
        private static final String SELECT_FIRST_ROW = "selectFirstRow";
        private static final String SELECT_FIRST_ROW_EXTEND = "selectFirstRowExtendSelection";
        private static final String SELECT_FIRST_ROW_CHANGE_LEAD = "selectFirstRowChangeLead";
        private static final String SELECT_LAST_ROW = "selectLastRow";
        private static final String SELECT_LAST_ROW_EXTEND = "selectLastRowExtendSelection";
        private static final String SELECT_LAST_ROW_CHANGE_LEAD = "selectLastRowChangeLead";
        private static final String SCROLL_UP = "scrollUp";
        private static final String SCROLL_UP_EXTEND = "scrollUpExtendSelection";
        private static final String SCROLL_UP_CHANGE_LEAD = "scrollUpChangeLead";
        private static final String SCROLL_DOWN = "scrollDown";
        private static final String SCROLL_DOWN_EXTEND = "scrollDownExtendSelection";
        private static final String SCROLL_DOWN_CHANGE_LEAD = "scrollDownChangeLead";
        private static final String SELECT_ALL = "selectAll";
        private static final String CLEAR_SELECTION = "clearSelection";
        private static final String ADD_TO_SELECTION = "addToSelection";
        private static final String TOGGLE_AND_ANCHOR = "toggleAndAnchor";
        private static final String EXTEND_TO = "extendTo";
        private static final String MOVE_SELECTION_TO = "moveSelectionTo";
        
        Actions(String name) {
            super(name);
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            String name = this.getName();
            JList<Object> list = (JList)actionEvent.getSource();
            BasicListUI basicListUI = (BasicListUI)BasicLookAndFeel.getUIOfType(list.getUI(), BasicListUI.class);
            if (name == "selectPreviousColumn") {
                this.changeSelection(list, 1, this.getNextColumnIndex(list, basicListUI, -1), -1);
            }
            else if (name == "selectPreviousColumnExtendSelection") {
                this.changeSelection(list, 2, this.getNextColumnIndex(list, basicListUI, -1), -1);
            }
            else if (name == "selectPreviousColumnChangeLead") {
                this.changeSelection(list, 0, this.getNextColumnIndex(list, basicListUI, -1), -1);
            }
            else if (name == "selectNextColumn") {
                this.changeSelection(list, 1, this.getNextColumnIndex(list, basicListUI, 1), 1);
            }
            else if (name == "selectNextColumnExtendSelection") {
                this.changeSelection(list, 2, this.getNextColumnIndex(list, basicListUI, 1), 1);
            }
            else if (name == "selectNextColumnChangeLead") {
                this.changeSelection(list, 0, this.getNextColumnIndex(list, basicListUI, 1), 1);
            }
            else if (name == "selectPreviousRow") {
                this.changeSelection(list, 1, this.getNextIndex(list, basicListUI, -1), -1);
            }
            else if (name == "selectPreviousRowExtendSelection") {
                this.changeSelection(list, 2, this.getNextIndex(list, basicListUI, -1), -1);
            }
            else if (name == "selectPreviousRowChangeLead") {
                this.changeSelection(list, 0, this.getNextIndex(list, basicListUI, -1), -1);
            }
            else if (name == "selectNextRow") {
                this.changeSelection(list, 1, this.getNextIndex(list, basicListUI, 1), 1);
            }
            else if (name == "selectNextRowExtendSelection") {
                this.changeSelection(list, 2, this.getNextIndex(list, basicListUI, 1), 1);
            }
            else if (name == "selectNextRowChangeLead") {
                this.changeSelection(list, 0, this.getNextIndex(list, basicListUI, 1), 1);
            }
            else if (name == "selectFirstRow") {
                this.changeSelection(list, 1, 0, -1);
            }
            else if (name == "selectFirstRowExtendSelection") {
                this.changeSelection(list, 2, 0, -1);
            }
            else if (name == "selectFirstRowChangeLead") {
                this.changeSelection(list, 0, 0, -1);
            }
            else if (name == "selectLastRow") {
                this.changeSelection(list, 1, list.getModel().getSize() - 1, 1);
            }
            else if (name == "selectLastRowExtendSelection") {
                this.changeSelection(list, 2, list.getModel().getSize() - 1, 1);
            }
            else if (name == "selectLastRowChangeLead") {
                this.changeSelection(list, 0, list.getModel().getSize() - 1, 1);
            }
            else if (name == "scrollUp") {
                this.changeSelection(list, 1, this.getNextPageIndex(list, -1), -1);
            }
            else if (name == "scrollUpExtendSelection") {
                this.changeSelection(list, 2, this.getNextPageIndex(list, -1), -1);
            }
            else if (name == "scrollUpChangeLead") {
                this.changeSelection(list, 0, this.getNextPageIndex(list, -1), -1);
            }
            else if (name == "scrollDown") {
                this.changeSelection(list, 1, this.getNextPageIndex(list, 1), 1);
            }
            else if (name == "scrollDownExtendSelection") {
                this.changeSelection(list, 2, this.getNextPageIndex(list, 1), 1);
            }
            else if (name == "scrollDownChangeLead") {
                this.changeSelection(list, 0, this.getNextPageIndex(list, 1), 1);
            }
            else if (name == "selectAll") {
                this.selectAll(list);
            }
            else if (name == "clearSelection") {
                this.clearSelection(list);
            }
            else if (name == "addToSelection") {
                int access$201 = BasicListUI.access$200(list.getSelectionModel().getLeadSelectionIndex(), list);
                if (!list.isSelectedIndex(access$201)) {
                    int anchorSelectionIndex = list.getSelectionModel().getAnchorSelectionIndex();
                    list.setValueIsAdjusting(true);
                    list.addSelectionInterval(access$201, access$201);
                    list.getSelectionModel().setAnchorSelectionIndex(anchorSelectionIndex);
                    list.setValueIsAdjusting(false);
                }
            }
            else if (name == "toggleAndAnchor") {
                int access$200 = BasicListUI.access$200(list.getSelectionModel().getLeadSelectionIndex(), list);
                if (list.isSelectedIndex(access$200)) {
                    list.removeSelectionInterval(access$200, access$200);
                }
                else {
                    list.addSelectionInterval(access$200, access$200);
                }
            }
            else if (name == "extendTo") {
                this.changeSelection(list, 2, BasicListUI.access$200(list.getSelectionModel().getLeadSelectionIndex(), list), 0);
            }
            else if (name == "moveSelectionTo") {
                this.changeSelection(list, 1, BasicListUI.access$200(list.getSelectionModel().getLeadSelectionIndex(), list), 0);
            }
        }
        
        public boolean isEnabled(Object o) {
            String name = this.getName();
            return (name != "selectPreviousColumnChangeLead" && name != "selectNextColumnChangeLead" && name != "selectPreviousRowChangeLead" && name != "selectNextRowChangeLead" && name != "selectFirstRowChangeLead" && name != "selectLastRowChangeLead" && name != "scrollUpChangeLead" && name != "scrollDownChangeLead") || (o != null && ((JList)o).getSelectionModel() instanceof DefaultListSelectionModel);
        }
        
        private void clearSelection(JList list) {
            list.clearSelection();
        }
        
        private void selectAll(JList list) {
            int size = list.getModel().getSize();
            if (size > 0) {
                ListSelectionModel selectionModel = list.getSelectionModel();
                int access$200 = BasicListUI.access$200(selectionModel.getLeadSelectionIndex(), list);
                if (selectionModel.getSelectionMode() == 0) {
                    if (access$200 == -1) {
                        int access$202 = BasicListUI.access$200(list.getMinSelectionIndex(), list);
                        access$200 = ((access$202 == -1) ? 0 : access$202);
                    }
                    list.setSelectionInterval(access$200, access$200);
                    list.ensureIndexIsVisible(access$200);
                }
                else {
                    list.setValueIsAdjusting(true);
                    int access$201 = BasicListUI.access$200(selectionModel.getAnchorSelectionIndex(), list);
                    list.setSelectionInterval(0, size - 1);
                    SwingUtilities2.setLeadAnchorWithoutSelection(selectionModel, access$201, access$200);
                    list.setValueIsAdjusting(false);
                }
            }
        }
        
        private int getNextPageIndex(JList list, int n) {
            if (list.getModel().getSize() == 0) {
                return -1;
            }
            Rectangle visibleRect = list.getVisibleRect();
            int access$200 = BasicListUI.access$200(list.getSelectionModel().getLeadSelectionIndex(), list);
            Rectangle rectangle = (access$200 == -1) ? new Rectangle() : list.getCellBounds(access$200, access$200);
            int n2;
            if (list.getLayoutOrientation() == 1 && list.getVisibleRowCount() <= 0) {
                if (!list.getComponentOrientation().isLeftToRight()) {
                    n = -n;
                }
                if (n < 0) {
                    visibleRect.x = rectangle.x + rectangle.width - visibleRect.width;
                    Point location3 = new Point(visibleRect.x - 1, rectangle.y);
                    n2 = list.locationToIndex(location3);
                    Rectangle r2 = list.getCellBounds(n2, n2);
                    if (visibleRect.intersects(r2)) {
                        location3.x = r2.x - 1;
                        n2 = list.locationToIndex(location3);
                        r2 = list.getCellBounds(n2, n2);
                    }
                    if (r2.y != rectangle.y) {
                        location3.x = r2.x + r2.width;
                        n2 = list.locationToIndex(location3);
                    }
                }
                else {
                    visibleRect.x = rectangle.x;
                    Point location2 = new Point(visibleRect.x + visibleRect.width, rectangle.y);
                    n2 = list.locationToIndex(location2);
                    Rectangle r = list.getCellBounds(n2, n2);
                    if (visibleRect.intersects(r)) {
                        location2.x = r.x + r.width;
                        n2 = list.locationToIndex(location2);
                        r = list.getCellBounds(n2, n2);
                    }
                    if (r.y != rectangle.y) {
                        location2.x = r.x - 1;
                        n2 = list.locationToIndex(location2);
                    }
                }
            }
            else if (n < 0) {
                Point point = new Point(rectangle.x, visibleRect.y);
                n2 = list.locationToIndex(point);
                if (access$200 <= n2) {
                    visibleRect.y = rectangle.y + rectangle.height - visibleRect.height;
                    point.y = visibleRect.y;
                    n2 = list.locationToIndex(point);
                    Rectangle rectangle3 = list.getCellBounds(n2, n2);
                    if (rectangle3.y < visibleRect.y) {
                        point.y = rectangle3.y + rectangle3.height;
                        n2 = list.locationToIndex(point);
                        rectangle3 = list.getCellBounds(n2, n2);
                    }
                    if (rectangle3.y >= rectangle.y) {
                        point.y = rectangle.y - 1;
                        n2 = list.locationToIndex(point);
                    }
                }
            }
            else {
                Point location = new Point(rectangle.x, visibleRect.y + visibleRect.height - 1);
                n2 = list.locationToIndex(location);
                Rectangle cellBounds = list.getCellBounds(n2, n2);
                if (cellBounds.y + cellBounds.height > visibleRect.y + visibleRect.height) {
                    location.y = cellBounds.y - 1;
                    int locationToIndex = list.locationToIndex(location);
                    list.getCellBounds(locationToIndex, locationToIndex);
                    n2 = Math.max(locationToIndex, access$200);
                }
                if (access$200 >= n2) {
                    visibleRect.y = rectangle.y;
                    location.y = visibleRect.y + visibleRect.height - 1;
                    n2 = list.locationToIndex(location);
                    Rectangle rectangle2 = list.getCellBounds(n2, n2);
                    if (rectangle2.y + rectangle2.height > visibleRect.y + visibleRect.height) {
                        location.y = rectangle2.y - 1;
                        n2 = list.locationToIndex(location);
                        rectangle2 = list.getCellBounds(n2, n2);
                    }
                    if (rectangle2.y <= rectangle.y) {
                        location.y = rectangle.y + rectangle.height;
                        n2 = list.locationToIndex(location);
                    }
                }
            }
            return n2;
        }
        
        private void changeSelection(JList list, int n, int lead, int n2) {
            if (lead >= 0 && lead < list.getModel().getSize()) {
                ListSelectionModel selectionModel = list.getSelectionModel();
                if (n == 0 && list.getSelectionMode() != 2) {
                    n = 1;
                }
                this.adjustScrollPositionIfNecessary(list, lead, n2);
                if (n == 2) {
                    int access$200 = BasicListUI.access$200(selectionModel.getAnchorSelectionIndex(), list);
                    if (access$200 == -1) {
                        access$200 = 0;
                    }
                    list.setSelectionInterval(access$200, lead);
                }
                else if (n == 1) {
                    list.setSelectedIndex(lead);
                }
                else {
                    ((DefaultListSelectionModel)selectionModel).moveLeadSelectionIndex(lead);
                }
            }
        }
        
        private void adjustScrollPositionIfNecessary(JList list, int n, int n2) {
            if (n2 == 0) {
                return;
            }
            Rectangle cellBounds = list.getCellBounds(n, n);
            Rectangle visibleRect = list.getVisibleRect();
            if (cellBounds != null && !visibleRect.contains(cellBounds)) {
                if (list.getLayoutOrientation() == 1 && list.getVisibleRowCount() <= 0) {
                    if (list.getComponentOrientation().isLeftToRight()) {
                        if (n2 > 0) {
                            int max2 = Math.max(0, cellBounds.x + cellBounds.width - visibleRect.width);
                            int locationToIndex4 = list.locationToIndex(new Point(max2, cellBounds.y));
                            Rectangle rectangle2 = list.getCellBounds(locationToIndex4, locationToIndex4);
                            if (rectangle2.x < max2 && rectangle2.x < cellBounds.x) {
                                rectangle2.x += rectangle2.width;
                                int locationToIndex5 = list.locationToIndex(rectangle2.getLocation());
                                rectangle2 = list.getCellBounds(locationToIndex5, locationToIndex5);
                            }
                            cellBounds = rectangle2;
                        }
                        cellBounds.width = visibleRect.width;
                    }
                    else if (n2 > 0) {
                        int x = cellBounds.x + visibleRect.width;
                        int locationToIndex3 = list.locationToIndex(new Point(x, cellBounds.y));
                        Rectangle cellBounds2 = list.getCellBounds(locationToIndex3, locationToIndex3);
                        if (cellBounds2.x + cellBounds2.width > x && cellBounds2.x > cellBounds.x) {
                            cellBounds2.width = 0;
                        }
                        cellBounds.x = Math.max(0, cellBounds2.x + cellBounds2.width - visibleRect.width);
                        cellBounds.width = visibleRect.width;
                    }
                    else {
                        cellBounds.x += Math.max(0, cellBounds.width - visibleRect.width);
                        cellBounds.width = Math.min(cellBounds.width, visibleRect.width);
                    }
                }
                else if (n2 > 0 && (cellBounds.y < visibleRect.y || cellBounds.y + cellBounds.height > visibleRect.y + visibleRect.height)) {
                    int max = Math.max(0, cellBounds.y + cellBounds.height - visibleRect.height);
                    int locationToIndex = list.locationToIndex(new Point(cellBounds.x, max));
                    Rectangle rectangle = list.getCellBounds(locationToIndex, locationToIndex);
                    if (rectangle.y < max && rectangle.y < cellBounds.y) {
                        rectangle.y += rectangle.height;
                        int locationToIndex2 = list.locationToIndex(rectangle.getLocation());
                        rectangle = list.getCellBounds(locationToIndex2, locationToIndex2);
                    }
                    cellBounds = rectangle;
                    cellBounds.height = visibleRect.height;
                }
                else {
                    cellBounds.height = Math.min(cellBounds.height, visibleRect.height);
                }
                list.scrollRectToVisible(cellBounds);
            }
            return;
        }
        
        private int getNextColumnIndex(JList list, BasicListUI basicListUI, int n) {
            if (list.getLayoutOrientation() != 0) {
                int access$200 = BasicListUI.access$200(list.getLeadSelectionIndex(), list);
                int size = list.getModel().getSize();
                if (access$200 == -1) {
                    return 0;
                }
                if (size == 1) {
                    return 0;
                }
                if (basicListUI == null || basicListUI.columnCount <= 1) {
                    return -1;
                }
                int access$201 = BasicListUI.access$400(basicListUI, access$200);
                int access$202 = BasicListUI.access$500(basicListUI, access$200);
                int n2 = access$201 + n;
                if (n2 >= basicListUI.columnCount || n2 < 0) {
                    return -1;
                }
                if (access$202 >= BasicListUI.access$600(basicListUI, n2)) {
                    return -1;
                }
                return BasicListUI.access$700(basicListUI, n2, access$202);
            }
            else {
                return -1;
            }
        }
        
        private int getNextIndex(JList list, BasicListUI basicListUI, int n) {
            int access$200 = BasicListUI.access$200(list.getLeadSelectionIndex(), list);
            int size = list.getModel().getSize();
            if (access$200 == -1) {
                if (size > 0) {
                    if (n > 0) {
                        access$200 = 0;
                    }
                    else {
                        access$200 = size - 1;
                    }
                }
            }
            else if (size == 1) {
                access$200 = 0;
            }
            else if (list.getLayoutOrientation() == 2) {
                if (basicListUI != null) {
                    access$200 += basicListUI.columnCount * n;
                }
            }
            else {
                access$200 += n;
            }
            return access$200;
        }
    }
    
    private class Handler implements FocusListener, KeyListener, ListDataListener, ListSelectionListener, MouseInputListener, PropertyChangeListener, DragRecognitionSupport.BeforeDrag
    {
        private String prefix;
        private String typedString;
        private long lastTime;
        private boolean dragPressDidSelection;
        
        private Handler() {
            this.prefix = "";
            this.typedString = "";
            this.lastTime = 0L;
        }
        
        @Override
        public void keyTyped(KeyEvent keyEvent) {
            JList<Object> list = (JList)keyEvent.getSource();
            ListModel<Object> model = list.getModel();
            if (model.getSize() == 0 || keyEvent.isAltDown() || BasicGraphicsUtils.isMenuShortcutKeyDown(keyEvent) || this.isNavigationKey(keyEvent)) {
                return;
            }
            boolean n = true;
            int keyChar = keyEvent.getKeyChar();
            long when = keyEvent.getWhen();
            int access$200 = BasicListUI.access$200(list.getLeadSelectionIndex(), BasicListUI.this.list);
            if (when - this.lastTime < BasicListUI.this.timeFactor) {
                this.typedString += (char)keyChar;
                if (this.prefix.length() == 1 && keyChar == this.prefix.charAt(0)) {
                    ++access$200;
                }
                else {
                    this.prefix = this.typedString;
                }
            }
            else {
                ++access$200;
                this.typedString = "" + (char)keyChar;
                this.prefix = this.typedString;
            }
            this.lastTime = when;
            if (access$200 < 0 || access$200 >= model.getSize()) {
                n = false;
                access$200 = 0;
            }
            int nextMatch = list.getNextMatch(this.prefix, access$200, Position.Bias.Forward);
            if (nextMatch >= 0) {
                list.setSelectedIndex(nextMatch);
                list.ensureIndexIsVisible(nextMatch);
            }
            else if (!(n == 0)) {
                int nextMatch2 = list.getNextMatch(this.prefix, 0, Position.Bias.Forward);
                if (nextMatch2 >= 0) {
                    list.setSelectedIndex(nextMatch2);
                    list.ensureIndexIsVisible(nextMatch2);
                }
            }
            return;
        }
        
        @Override
        public void keyPressed(KeyEvent keyEvent) {
            if (this.isNavigationKey(keyEvent)) {
                this.prefix = "";
                this.typedString = "";
                this.lastTime = 0L;
            }
        }
        
        @Override
        public void keyReleased(KeyEvent keyEvent) {
        }
        
        private boolean isNavigationKey(KeyEvent anEvent) {
            InputMap inputMap = BasicListUI.this.list.getInputMap(1);
            KeyStroke keyStrokeForEvent = KeyStroke.getKeyStrokeForEvent(anEvent);
            return inputMap != null && inputMap.get(keyStrokeForEvent) != null;
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            String propertyName = propertyChangeEvent.getPropertyName();
            if (propertyName == "model") {
                ListModel<Object> listModel = (ListModel)propertyChangeEvent.getOldValue();
                ListModel<Object> listModel2 = (ListModel)propertyChangeEvent.getNewValue();
                if (listModel != null) {
                    listModel.removeListDataListener(BasicListUI.this.listDataListener);
                }
                if (listModel2 != null) {
                    listModel2.addListDataListener(BasicListUI.this.listDataListener);
                }
                BasicListUI this$10 = BasicListUI.this;
                this$10.updateLayoutStateNeeded |= 0x1;
                BasicListUI.access$900(BasicListUI.this);
            }
            else if (propertyName == "selectionModel") {
                ListSelectionModel listSelectionModel = (ListSelectionModel)propertyChangeEvent.getOldValue();
                ListSelectionModel listSelectionModel2 = (ListSelectionModel)propertyChangeEvent.getNewValue();
                if (listSelectionModel != null) {
                    listSelectionModel.removeListSelectionListener(BasicListUI.this.listSelectionListener);
                }
                if (listSelectionModel2 != null) {
                    listSelectionModel2.addListSelectionListener(BasicListUI.this.listSelectionListener);
                }
                BasicListUI this$9 = BasicListUI.this;
                this$9.updateLayoutStateNeeded |= 0x1;
                BasicListUI.access$900(BasicListUI.this);
            }
            else if (propertyName == "cellRenderer") {
                BasicListUI this$8 = BasicListUI.this;
                this$8.updateLayoutStateNeeded |= 0x40;
                BasicListUI.access$900(BasicListUI.this);
            }
            else if (propertyName == "font") {
                BasicListUI this$7 = BasicListUI.this;
                this$7.updateLayoutStateNeeded |= 0x4;
                BasicListUI.access$900(BasicListUI.this);
            }
            else if (propertyName == "prototypeCellValue") {
                BasicListUI this$6 = BasicListUI.this;
                this$6.updateLayoutStateNeeded |= 0x20;
                BasicListUI.access$900(BasicListUI.this);
            }
            else if (propertyName == "fixedCellHeight") {
                BasicListUI this$5 = BasicListUI.this;
                this$5.updateLayoutStateNeeded |= 0x10;
                BasicListUI.access$900(BasicListUI.this);
            }
            else if (propertyName == "fixedCellWidth") {
                BasicListUI this$4 = BasicListUI.this;
                this$4.updateLayoutStateNeeded |= 0x8;
                BasicListUI.access$900(BasicListUI.this);
            }
            else if (propertyName == "selectionForeground") {
                BasicListUI.this.list.repaint();
            }
            else if (propertyName == "selectionBackground") {
                BasicListUI.this.list.repaint();
            }
            else if ("layoutOrientation" == propertyName) {
                BasicListUI this$3 = BasicListUI.this;
                this$3.updateLayoutStateNeeded |= 0x80;
                BasicListUI.this.layoutOrientation = BasicListUI.this.list.getLayoutOrientation();
                BasicListUI.access$900(BasicListUI.this);
            }
            else if ("visibleRowCount" == propertyName) {
                if (BasicListUI.this.layoutOrientation != 0) {
                    BasicListUI this$2 = BasicListUI.this;
                    this$2.updateLayoutStateNeeded |= 0x80;
                    BasicListUI.access$900(BasicListUI.this);
                }
            }
            else if ("componentOrientation" == propertyName) {
                BasicListUI.this.isLeftToRight = BasicListUI.this.list.getComponentOrientation().isLeftToRight();
                BasicListUI this$0 = BasicListUI.this;
                this$0.updateLayoutStateNeeded |= 0x400;
                BasicListUI.access$900(BasicListUI.this);
                SwingUtilities.replaceUIInputMap(BasicListUI.this.list, 0, BasicListUI.this.getInputMap(0));
            }
            else if ("List.isFileList" == propertyName) {
                BasicListUI.access$1200(BasicListUI.this);
                BasicListUI.access$900(BasicListUI.this);
            }
            else if ("dropLocation" == propertyName) {
                this.repaintDropLocation((JList.DropLocation)propertyChangeEvent.getOldValue());
                this.repaintDropLocation(BasicListUI.this.list.getDropLocation());
            }
        }
        
        private void repaintDropLocation(JList.DropLocation dropLocation) {
            if (dropLocation == null) {
                return;
            }
            Rectangle r;
            if (dropLocation.isInsert()) {
                r = BasicListUI.access$1300(BasicListUI.this, dropLocation);
            }
            else {
                r = BasicListUI.access$1400(BasicListUI.this, BasicListUI.this.list, dropLocation.getIndex());
            }
            if (r != null) {
                BasicListUI.this.list.repaint(r);
            }
            return;
        }
        
        @Override
        public void intervalAdded(ListDataEvent listDataEvent) {
            BasicListUI.this.updateLayoutStateNeeded = 1;
            int min = Math.min(listDataEvent.getIndex0(), listDataEvent.getIndex1());
            int max = Math.max(listDataEvent.getIndex0(), listDataEvent.getIndex1());
            ListSelectionModel selectionModel = BasicListUI.this.list.getSelectionModel();
            if (selectionModel != null) {
                selectionModel.insertIndexInterval(min, max - min + 1, true);
            }
            BasicListUI.access$900(BasicListUI.this);
        }
        
        @Override
        public void intervalRemoved(ListDataEvent listDataEvent) {
            BasicListUI.this.updateLayoutStateNeeded = 1;
            ListSelectionModel selectionModel = BasicListUI.this.list.getSelectionModel();
            if (selectionModel != null) {
                selectionModel.removeIndexInterval(listDataEvent.getIndex0(), listDataEvent.getIndex1());
            }
            BasicListUI.access$900(BasicListUI.this);
        }
        
        @Override
        public void contentsChanged(ListDataEvent listDataEvent) {
            BasicListUI.this.updateLayoutStateNeeded = 1;
            BasicListUI.access$900(BasicListUI.this);
        }
        
        @Override
        public void valueChanged(ListSelectionEvent listSelectionEvent) {
            BasicListUI.this.maybeUpdateLayoutState();
            int size = BasicListUI.this.list.getModel().getSize();
            Rectangle cellBounds = BasicListUI.this.getCellBounds(BasicListUI.this.list, Math.min(size - 1, Math.max(listSelectionEvent.getFirstIndex(), 0)), Math.min(size - 1, Math.max(listSelectionEvent.getLastIndex(), 0)));
            if (cellBounds != null) {
                BasicListUI.this.list.repaint(cellBounds.x, cellBounds.y, cellBounds.width, cellBounds.height);
            }
        }
        
        @Override
        public void mouseClicked(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseEntered(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseExited(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mousePressed(MouseEvent me) {
            if (SwingUtilities2.shouldIgnore(me, BasicListUI.this.list)) {
                return;
            }
            int dragEnabled = BasicListUI.this.list.getDragEnabled();
            boolean n = true;
            if (!(dragEnabled == 0)) {
                int loc2IndexFileList = SwingUtilities2.loc2IndexFileList(BasicListUI.this.list, me.getPoint());
                if (loc2IndexFileList != -1 && DragRecognitionSupport.mousePressed(me)) {
                    this.dragPressDidSelection = false;
                    if (BasicGraphicsUtils.isMenuShortcutKeyDown(me)) {
                        return;
                    }
                    if (!me.isShiftDown() && BasicListUI.this.list.isSelectedIndex(loc2IndexFileList)) {
                        BasicListUI.this.list.addSelectionInterval(loc2IndexFileList, loc2IndexFileList);
                        return;
                    }
                    n = false;
                    this.dragPressDidSelection = true;
                }
            }
            else {
                BasicListUI.this.list.setValueIsAdjusting(true);
            }
            if (!(n == 0)) {
                SwingUtilities2.adjustFocus(BasicListUI.this.list);
            }
            this.adjustSelection(me);
            return;
        }
        
        private void adjustSelection(MouseEvent mouseEvent) {
            int loc2IndexFileList = SwingUtilities2.loc2IndexFileList(BasicListUI.this.list, mouseEvent.getPoint());
            if (loc2IndexFileList < 0) {
                if (BasicListUI.this.isFileList && mouseEvent.getID() == 501 && (!mouseEvent.isShiftDown() || BasicListUI.this.list.getSelectionMode() == 0)) {
                    BasicListUI.this.list.clearSelection();
                }
            }
            else {
                int access$200 = BasicListUI.access$200(BasicListUI.this.list.getAnchorSelectionIndex(), BasicListUI.this.list);
                int selectedIndex;
                if (access$200 == -1) {
                    access$200 = 0;
                    selectedIndex = false;
                }
                else {
                    selectedIndex = BasicListUI.this.list.isSelectedIndex(access$200);
                }
                if (BasicGraphicsUtils.isMenuShortcutKeyDown(mouseEvent)) {
                    if (mouseEvent.isShiftDown()) {
                        if (!(selectedIndex == 0)) {
                            BasicListUI.this.list.addSelectionInterval(access$200, loc2IndexFileList);
                        }
                        else {
                            BasicListUI.this.list.removeSelectionInterval(access$200, loc2IndexFileList);
                            if (BasicListUI.this.isFileList) {
                                BasicListUI.this.list.addSelectionInterval(loc2IndexFileList, loc2IndexFileList);
                                BasicListUI.this.list.getSelectionModel().setAnchorSelectionIndex(access$200);
                            }
                        }
                    }
                    else if (BasicListUI.this.list.isSelectedIndex(loc2IndexFileList)) {
                        BasicListUI.this.list.removeSelectionInterval(loc2IndexFileList, loc2IndexFileList);
                    }
                    else {
                        BasicListUI.this.list.addSelectionInterval(loc2IndexFileList, loc2IndexFileList);
                    }
                }
                else if (mouseEvent.isShiftDown()) {
                    BasicListUI.this.list.setSelectionInterval(access$200, loc2IndexFileList);
                }
                else {
                    BasicListUI.this.list.setSelectionInterval(loc2IndexFileList, loc2IndexFileList);
                }
            }
        }
        
        @Override
        public void dragStarting(MouseEvent mouseEvent) {
            if (BasicGraphicsUtils.isMenuShortcutKeyDown(mouseEvent)) {
                int loc2IndexFileList = SwingUtilities2.loc2IndexFileList(BasicListUI.this.list, mouseEvent.getPoint());
                BasicListUI.this.list.addSelectionInterval(loc2IndexFileList, loc2IndexFileList);
            }
        }
        
        @Override
        public void mouseDragged(MouseEvent me) {
            if (SwingUtilities2.shouldIgnore(me, BasicListUI.this.list)) {
                return;
            }
            if (BasicListUI.this.list.getDragEnabled()) {
                DragRecognitionSupport.mouseDragged(me, this);
                return;
            }
            if (me.isShiftDown() || BasicGraphicsUtils.isMenuShortcutKeyDown(me)) {
                return;
            }
            int locationToIndex = BasicListUI.this.locationToIndex(BasicListUI.this.list, me.getPoint());
            if (locationToIndex != -1) {
                if (BasicListUI.this.isFileList) {
                    return;
                }
                Rectangle cellBounds = BasicListUI.this.getCellBounds(BasicListUI.this.list, locationToIndex, locationToIndex);
                if (cellBounds != null) {
                    BasicListUI.this.list.scrollRectToVisible(cellBounds);
                    BasicListUI.this.list.setSelectionInterval(locationToIndex, locationToIndex);
                }
            }
            return;
        }
        
        @Override
        public void mouseMoved(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseReleased(MouseEvent me) {
            if (SwingUtilities2.shouldIgnore(me, BasicListUI.this.list)) {
                return;
            }
            if (BasicListUI.this.list.getDragEnabled()) {
                MouseEvent mouseReleased = DragRecognitionSupport.mouseReleased(me);
                if (mouseReleased != null) {
                    SwingUtilities2.adjustFocus(BasicListUI.this.list);
                    if (!this.dragPressDidSelection) {
                        this.adjustSelection(mouseReleased);
                    }
                }
            }
            else {
                BasicListUI.this.list.setValueIsAdjusting(false);
            }
            return;
        }
        
        protected void repaintCellFocus() {
            int access$200 = BasicListUI.access$200(BasicListUI.this.list.getLeadSelectionIndex(), BasicListUI.this.list);
            if (access$200 != -1) {
                Rectangle cellBounds = BasicListUI.this.getCellBounds(BasicListUI.this.list, access$200, access$200);
                if (cellBounds != null) {
                    BasicListUI.this.list.repaint(cellBounds.x, cellBounds.y, cellBounds.width, cellBounds.height);
                }
            }
        }
        
        @Override
        public void focusGained(FocusEvent focusEvent) {
            this.repaintCellFocus();
        }
        
        @Override
        public void focusLost(FocusEvent focusEvent) {
            this.repaintCellFocus();
        }
    }
    
    static class ListTransferHandler extends TransferHandler implements UIResource
    {
        @Override
        protected Transferable createTransferable(JComponent component) {
            if ((component instanceof JList)) {
                Object[] selectedValues = ((JList)component).getSelectedValues();
                if (selectedValues == null || selectedValues.length == 0) {
                    return null;
                }
                StringBuffer sb = new StringBuffer();
                StringBuffer sb2 = new StringBuffer();
                sb2.append("<html>\n<body>\n<ul>\n");
                for (int i = 0; i < selectedValues.length; ++i) {
                    Object o = selectedValues[i];
                    String s = (o == null) ? "" : o.toString();
                    sb.append(s + "\n");
                    sb2.append("  <li>" + s + "\n");
                }
                sb.deleteCharAt(sb.length() - 1);
                sb2.append("</ul>\n</body>\n</html>");
                return new BasicTransferable(sb.toString(), sb2.toString());
            }
            else {
                return null;
            }
        }
        
        @Override
        public int getSourceActions(JComponent component) {
            return 1;
        }
    }
}
